#include <linux/config.h>

#include <linux/proc_fs.h>
#include <linux/ioport.h>
#include <linux/bios32.h>
#include <asm/io.h>
#include <linux/pci.h>
#include "lm75.h"

static int Init_LM75(void);
static void Cleanup_LM75(void);
static int print_lm75_values (int buf);


static int read_smbus_word(int addr);

unsigned short PIIX4_smba = 0;


/* Time killer work around */
static void our_usleep(int wait_time);


/* This kill some time in a baaaaaad way. :') */
static void our_usleep(int wait_time) {
long killtime;
long temp;
 for (; wait_time > 0; wait_time--) {
         for (killtime=0; killtime<=4000; killtime++) {
                temp=0xAABC; temp= (temp << 13);
         }
  }
}

/*
 *      Initialize LM75/SMBus code.
 */
int Init_LM75(void){
int error_return=0;
unsigned char  PIIX4_bus, PIIX4_devfn;

    if (pcibios_present() == 0) {
        printk("LM78: Error: No PCI-bus found!\n");
        error_return=1;
    } else {    
        if(pcibios_find_device(PCI_VENDOR_ID_INTEL, PCI_DEVICE_ID_INTEL_82371AB_0, 0,
                               &PIIX4_bus, &PIIX4_devfn)) {
            printk("LM78: Error: Can<B4>t detect PIIX4!\n");
            error_return=1;
        } else {
            pcibios_read_config_word(PIIX4_bus, PIIX4_devfn | 3, 0x90, &PIIX4_smba);
            PIIX4_smba &= 0xfff0;   
            if (check_region(PIIX4_smba, 7))
                return 0; 
            request_region(PIIX4_smba, 7, "lm78");
        }   
    }   
    if (error_return)
        printk("LM78: SMBus setup failed.  LM75 readings will be disabled.\n");
/* This line is for debugging */
    printk("PIIX4_smba = 0x%X\n",PIIX4_smba);
    return error_return;
}


/*
 *      Find and read LM75 at given address.
 */
int read_smbus_word(int addr) {

int HOST_BUSY;
long timeout=0;
int result=0;

/* Setup protocol type, address, device command, data, and control bits */
    outb_p( 0x00, SMBHSTDAT0);
    outb_p( 0x00, SMBHSTDAT1);
    outb_p( 0x0C, SMBHSTCNT); /* Read word */
    if (inb_p(SMBHSTSTS) != 0x00) {
        outb_p( inb(SMBHSTSTS), SMBHSTSTS);
    }
    outb_p( (addr << 1) | 0x01, SMBHSTADD);
/* address of possible thermal sensors are from 1
 001000x to 1001111x ( x = r/w bit ) */

/* Set the start bit */
    outb_p(inb(SMBHSTCNT) | 0x040, SMBHSTCNT); 
/* start the transaction by setting bit 6 */

/* Wait for a fraction of a second! */
   our_usleep(100);

/* Pole Host_busy bit */
   HOST_BUSY=inb_p(SMBHSTSTS) & 0x01;
   while (((inb_p(SMBHSTSTS) & 0x01)) & (timeout < MAX_TIMEOUT)) {
/* Wait for a while */
        our_usleep(50);
   }
   if (! (inb(SMBHSTSTS) & 0x01)) {
        result=(inb(SMBHSTDAT0) << 8) | inb(SMBHSTDAT1);
   }

/* Sense for no device at the address */
   if (inb_p(SMBHSTSTS) == 0x04)  {
        result=-1;
   }
   return result;
}

int print_lm75_values (int buf) {

int i;
int len=0;
int value;

    /* Probe each possible valid LM75 address and report the temp for each found */
        for (i=0x048; i<=0x04F; i++) { 
	        value=read_smbus_word(i) >> 7;
	        if (value != -1) {
	           len = sprintf(buf+len, "LM75 at address 0x%X reports %d.%d degrees C\n",i,
			value  >> 1, (value & 1) * 5);
	        }
	}
	return len;
}

/*
 *      Clean-up the LM75/SMBus code.
 */
void Cleanup_LM75(void){
        release_region(PIIX4_smba, 7); 
}
